--[[
Author: Starinnia
CPR is a combo points display addon based on Funkydude's BasicComboPoints
Combo.lua - A module for tracking combo points
$Date: 2010-07-19 21:43:01 +0000 (Mon, 19 Jul 2010) $
$Revision: 94 $
Project Version: r94
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2007-2010 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

local cpr = LibStub("AceAddon-3.0"):GetAddon("ComboPointsRedux")
local modName = "ComboPoints"
local ComboModule = cpr:NewModule(modName, "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ComboPointsRedux")
local LSM = LibStub("LibSharedMedia-3.0")
ComboModule.abbrev = "CP"

function ComboModule:OnInitialize()
	self.graphics = cpr:MakeGraphicsFrame(modName)
	self.text = cpr:MakeTextFrame(modName)
end

function ComboModule:OnEnable()
	if not cpr.db.profile.modules[modName].disableGraphics then
		self.graphics:Show()
	end
	if not cpr.db.profile.modules[modName].disableText then
		self.text:Show()
	end
	
	self:RegisterEvent("UNIT_COMBO_POINTS")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "UNIT_COMBO_POINTS")
end

function ComboModule:OnDisable()
	self.graphics:Hide()
	self.text:Hide()
	self:UnregisterEvent("UNIT_COMBO_POINTS")
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
end

function ComboModule:UNIT_COMBO_POINTS()
	local points = GetComboPoints(UnitHasVehicleUI("player") and "vehicle" or "player", "target")
	if UnitIsDead("target") then points = 0 end
	local r, g, b = cpr:GetColorByPoints(modName, points)
	
	if points == 0 then
		for i = 1, 5 do
			self.graphics.points[i]:Hide()
		end
		points = ""
	else
		for i = points, 1, -1 do
			self.graphics.points[i].icon:SetVertexColor(r, g, b)
			self.graphics.points[i]:Show()
		end
	end
	self.text:SetNumPoints(points)
end

--Set handler for the module options
function ComboModule:Set(info, ...)
	if info.type == "color" then
		local color = cpr.db.profile.modules[modName][info.arg]
		color[1], color[2], color[3] = ...
	else
		cpr.db.profile.modules[modName][info.arg] = ...
	end
	self:UpdateSettings()
end

--Get handler for the module options
function ComboModule:Get(info, ...)
	if info.type == "color" then
		return unpack(cpr.db.profile.modules[modName][info.arg])
	else
		return cpr.db.profile.modules[modName][info.arg]
	end
end

local basepath = [[Interface\Addons\ComboPointsRedux\Icons\]]
--Update is invoked by the Set handler after a change
function ComboModule:UpdateSettings(isCallback)
	local db = cpr.db.profile.modules[modName]
	
	--graphics
	if not db.disableGraphics then
		--change icon textures
		--update icon Alpha
		for i = 1, 5 do
			self.graphics.points[i].icon:SetTexture(basepath..db.icon)
			self.graphics.points[i]:SetAlpha(db.graphicsAlpha)
			self.graphics.points[i]:SetWidth(25*db.scale)
			self.graphics.points[i]:SetHeight(25*db.scale)
			self.graphics.points[i]:ClearAllPoints()
		end
		
		--adjust container scale
		if db.orientation == "v" then
			self.graphics:SetHeight(145*db.scale)
			self.graphics:SetWidth(25*db.scale)
		else
			self.graphics:SetWidth(145*db.scale)
			self.graphics:SetHeight(25*db.scale)
		end
		
		--adjust for orientation changes (this updates spacing too)
		local offset = db.spacing*db.scale
		if db.orientation == "v" then
			self.graphics.points[1]:SetPoint("BOTTOM", self.graphics, "BOTTOM", 0, 0)
			self.graphics.points[2]:SetPoint("BOTTOM", self.graphics.points[1], "TOP", 0, offset)
			self.graphics.points[3]:SetPoint("BOTTOM", self.graphics.points[2], "TOP", 0, offset)
			self.graphics.points[4]:SetPoint("BOTTOM", self.graphics.points[3], "TOP", 0, offset)
			self.graphics.points[5]:SetPoint("BOTTOM", self.graphics.points[4], "TOP", 0, offset)
		else
			self.graphics.points[1]:SetPoint("LEFT", self.graphics, "LEFT", 0, 0)
			self.graphics.points[2]:SetPoint("LEFT", self.graphics.points[1], "RIGHT", offset, 0)
			self.graphics.points[3]:SetPoint("LEFT", self.graphics.points[2], "RIGHT", offset, 0)
			self.graphics.points[4]:SetPoint("LEFT", self.graphics.points[3], "RIGHT", offset, 0)
			self.graphics.points[5]:SetPoint("LEFT", self.graphics.points[4], "RIGHT", offset, 0)
		end
		
		--update strata
		self.graphics:SetFrameStrata(db.strata)
		
		--update locked status
		if not cpr.db.profile.locked then
			self.graphics:SetBackdropColor(1,1,1,1)
			self.graphics:EnableMouse(true)
			self.graphics:SetMovable(true)
			self.graphics.points[1]:Show()
			self.graphics.points[2]:Show()
		else
			self.graphics:EnableMouse(false)
			self.graphics:SetMovable(false)
		end
	end
	
	--text
	if not db.disableText then
		--update font, font size, alpha and strata
		self.text.count:SetShadowColor(0, 0, 0, db.textAlpha)
		self.text.count:SetFont(LSM:Fetch("font", db.font), db.fontsize)
		self.text.count:SetTextColor(unpack(db.textColor))
		self.text.count:SetAlpha(db.textAlpha)
		self.text:SetFrameStrata(db.textStrata)
		
		--update locked status
		if not cpr.db.profile.locked then
			self.text:SetBackdropColor(1,1,1,1)
			self.text:EnableMouse(true)
			self.text:SetMovable(true)
			self.text.count:SetText(self.abbrev)
		else
			self.text:EnableMouse(false)
			self.text:SetMovable(false)
		end
	end
	
	--if this call is from a profile callback then we also need to reset the anchors
	if isCallback then
		if db.textPos.x then
			self.text:SetPoint(db.textPos.point, UIParent, db.textPos.anchor, db.textPos.x, db.textPos.y)
		else
			self.text:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		end
		
		if db.graphicsPos.x then
			self.graphics:SetPoint(db.graphicsPos.point, UIParent, db.graphicsPos.anchor, db.graphicsPos.x, db.graphicsPos.y)
		else
			self.graphics:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		end
	end
end

function ComboModule:ResetSettings()
	local db = cpr.db.profile.modules[modName]
	self.text:SetPoint("CENTER", UIParent, "CENTER")
	self.graphics:SetPoint("CENTER", UIParent, "CENTER")
	
	self.graphics:SetWidth(145*db.scale)
	self.graphics:SetHeight(25*db.scale)
	
	for i = 1, 5 do
		self.graphics.points[i].icon:SetTexture(basepath..db.icon)
		self.graphics.points[i]:SetWidth(25*db.scale)
		self.graphics.points[i]:SetHeight(25*db.scale)
		self.graphics.points[i]:ClearAllPoints()
	end
	
	local offset = db.spacing*db.scale
	self.graphics.points[1]:SetPoint("LEFT", self.graphics, "LEFT", 0, 0)
	self.graphics.points[2]:SetPoint("LEFT", self.graphics.points[1], "RIGHT", offset, 0)
	self.graphics.points[3]:SetPoint("LEFT", self.graphics.points[2], "RIGHT", offset, 0)
	self.graphics.points[4]:SetPoint("LEFT", self.graphics.points[3], "RIGHT", offset, 0)
	self.graphics.points[5]:SetPoint("LEFT", self.graphics.points[4], "RIGHT", offset, 0)
	
	self.graphics:SetBackdropColor(1,1,1,1)
	self.graphics:EnableMouse(true)
	self.graphics:SetMovable(true)
	self.graphics.points[1]:Show()
	self.graphics.points[2]:Show()
	
	self.text:SetBackdropColor(1,1,1,1)
	self.text:EnableMouse(true)
	self.text:SetMovable(true)
	self.text.count:SetText(self.abbrev)
end

function ComboModule:GetOptions()
	return {
		type = "group",
		name = "Combo Points",
		handler = self,
		disabled = function() return not cpr.db.profile.modules[modName].enabled end,
		order = 2,
		args = {
			disablegraphics = {
				type = "toggle",
				name = L["Disable Graphics"],
				desc = L["Select whether or not to show the combo points graphics."],
				descStyle = "inline",
				width = "full",
				arg = "disableGraphics",
				get = "Get",
				set = function(info, value)
					cpr.db.profile.modules[modName].disableGraphics = value
					if value then
						self.graphics:Hide()
					else
						self.graphics:Show()
					end
				end,
				order = 1,
			},
			disabletext = {
				type = "toggle",
				name = L["Disable Text"],
				desc = L["Select whether or not to show the combo points text counter."],
				descStyle = "inline",
				width = "full",
				arg = "disableText",
				get = "Get",
				set = function(info, value)
					cpr.db.profile.modules[modName].disableText = value
					if value then
						self.text:Hide()
					else
						self.text:Show()
					end
				end,
				order = 2,
			},
			graphics = {
				type = "group",
				name = L["Combo Points Graphics"],
				inline = true,
				order = 3,
				disabled = function() return cpr.db.profile.modules[modName].disableGraphics end,
				args = {
					onecolor = {
						type = 'color',
						name = L["One Point Color"],
						desc = L["Set the color to be used when you have one combo point."],
						arg = "oneColor",
						get = "Get",
						set = "Set",
						order = 1,
					},
					twocolor = {
						type = 'color',
						name = L["Two Points Color"],
						desc = L["Set the color to be used when you have two combo points."],
						arg = "twoColor",
						get = "Get",
						set = "Set",
						order = 2,
					},
					threecolor = {
						type = 'color',
						name = L["Three Points Color"],
						desc = L["Set the color to be used when you have three combo points."],
						arg = "threeColor",
						get = "Get",
						set = "Set",
						order = 3,
					},
					fourcolor = {
						type = 'color',
						name = L["Four Points Color"],
						desc = L["Set the color to be used when you have four combo points."],
						arg = "fourColor",
						get = "Get",
						set = "Set",
						order = 4,
					},
					fivecolor = {
						type = 'color',
						name = L["Five Points Color"],
						desc = L["Set the color to be used when you have five combo points."],
						arg = "fiveColor",
						get = "Get",
						set = "Set",
						order = 5,
					},
					icon = {
						type = "select",
						name = L["Icon"],
						desc = L["Select the icon to be used to represent your combo points."],
						arg = "icon",
						get = "Get",
						set = "Set",
						values = { square = L["Square"], circle = L["Circle"], diamond = L["Diamond"] },
						order = 6,
					},
					orientation = {
						type = "select",
						name = L["Orientation"],
						desc = L["Select the orientation of the graphical display (horizontal or vertical)."],
						arg = "orientation",
						get = "Get",
						set = "Set",
						values = { h = L["Horizontal"], v = L["Vertical"] },
						order = 7,
					},
					strata = {
						type = 'select',
						name = L["Graphics Strata"],
						desc = L["Set the strata level of the graphics frame."],
						get = "Get",
						set = "Set",
						arg = "strata",
						values = {["FULLSCREEN"] = "Fullscreen", ["HIGH"] = "High", ["MEDIUM"] = "Medium", ["LOW"] = "Low", ["BACKGROUND"] = "Background",},
						order = 8,
					},
					alpha = {
						type = 'range',
						name = L["Icon Alpha"],
						desc = L["Set the alpha of the icons."],
						arg = "alpha",
						get = "Get",
						set = "Set",
						min = 0.0,
						max = 1.0,
						step = 0.01,
						order = 9,
					},
					scale = {
						type = 'range',
						name = L["Scale"],
						desc = L["Set the scale of the icons."],
						arg = "scale",
						get = "Get",
						set = "Set",
						min = 0.1,
						max = 1.5,
						step = 0.01,
						order = 10,
					},
					spacing = {
						type = 'range',
						name = L["Icon Spacing"],
						desc = L["Set the spacing between the icons."],
						arg = "spacing",
						get = "Get",
						set = "Set",
						min = -2,
						max = 10,
						step = 1,
						order = 11,
					},
				},
			},
			text = {
				type = "group",
				name = L["Combo Points Text"],
				inline = true,
				order = 4,
				disabled = function() return cpr.db.profile.modules[modName].disableText end,
				args = {
					font = {
						type = 'select',
						name = L["Font"],
						desc = L["Select the font for the text display."],
						arg = "font",
						get = "Get",
						set = "Set",
						dialogControl = "LSM30_Font",
						values = AceGUIWidgetLSMlists.font,
						order = 1,
					},
					fontsize = {
						type = 'range',
						name = L["Font Size"],
						desc = L["Set the font size for the text display."],
						arg = "fontsize",
						get = "Get",
						set = "Set",
						min = 1,
						max = 36,
						step = 1,
						order = 2,
					},
					textcolor = {
						type = 'color',
						name = L["Text Color"],
						desc = L["Set the color for the text display."],
						arg = "textColor",
						get = "Get",
						set = "Set",
						order = 3,
					},
					textalpha = {
						type = 'range',
						name = L["Text Alpha"],
						desc = L["Set the alpha of the text display."],
						arg = "textAlpha",
						get = "Get",
						set = "Set",
						min = 0.0,
						max = 1.0,
						step = 0.01,
						order = 4,
					},
					textstrata = {
						type = 'select',
						name = L["Text Strata"],
						desc = L["Set the strata level of the text frame."],
						get = "Get",
						set = "Set",
						arg = "textStrata",
						values = {["FULLSCREEN"] = "Fullscreen", ["HIGH"] = "High", ["MEDIUM"] = "Medium", ["LOW"] = "Low", ["BACKGROUND"] = "Background",},
						order = 5,
					},
				},
			},
		},
	}
end
